/*
 * Receiver "Armstrong Initial", 2m. 
 * 
 * 200 Channels in EEPROM. Scan channels and scan frequency. 
 * All functions for scan, channels and sleep mode are written by me, S. Malkin 
 * Xosc = 27 MHz
 * Buttons: read (12), write (11), select (10), run (9), calibration (5).
 * 04.09.2019
 * 
 * 
 * 
  A part of this program is taken from Jason Mildrum, NT7S.
  All extra functions are written by me, Rob Engberts PA0RWE

  References:
  http://nt7s.com/
  http://sq9nje.pl/
  http://ak2b.blogspot.com/
  http://pa0rwe.nl/?page_id=804

    SI5351_VFO control program for Arduino NANO
    Copyright PA0RWE Rob Engberts

    Using the Si5351 library by Jason Mildrun nt7s

    Functions:
    - CLK0 - Tx frequency = Display frequency
    - CLK1 - Rx / RIT frequency = Tx +/- BFO (upper- or lower mixing)
             When RIT active, RIT frequency is displayed and is tunable.
             When RIT is inactive Rx = Tx +/- BFO
    - CLK2 - BFO frequency, tunable

    - Stepsize:  select (pushbutton)
    - Calibrate: (pushbutton) calculates difference between X-tal and measured
                 x-tal frequency, to correct x-tal frequency.
    - Selection: (pushbutton) Switch between TRx and BFO mode
    - RIT switch: tunable Rx frequency, while Tx frequency not changed
    - Programming PIC by ICSP

    Si5351 settings: I2C address is in the .h file
                     X-tal freq is in the .h file but set in line 345


***************************************************************************
    02-04-2015   0.1    Start building program based on the PIC version

***************************************************************************
   Includes
**************************************************************************/

#include <Rotary.h>
#include <si5351.h>
#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <EEPROM.h>
#include<TimerOne.h>
#include <avr/sleep.h>
/**************************************************************************
   Definitions BUTTONS
**************************************************************************/
#define ENCODER_A     2       // Encoder pin A INT0/PCINT18 D2
#define ENCODER_B     3       // Encoder pin B INT1/PCINT19 D3
#define ENCODER_BTN   4       // Encoder pushbutton D4

#define OLED_RESET    8       // OLED reset //not used

#define Calibrbtn     5       // Calibrate

#define RIT_Switch    6       // RIT Switch // not used
#define TX_Switch     7       // Select TRx or BFO //not used

////////////SCAN BUTTONS////////////
#define scanstart     9       // run/stop scan (even without choose frequencies, but F(start) must be lower than F(end))
#define scanfreq     10       // select scan frequencies: F(start), F(end) and start to scan
/////////// ! SCAN BUTTONS///////////

////////////CHANNELS BUTTONS////////
#define channels_write_button     11  // WRITE channels TO EEPROM
#define channels_read_button     12   // READ channels from EEPROM
//////////// ! CHANNELS///////

// ! BUTTONS//

// LOWER AND UPPER FREQUENCIES
//#define F_MIN        1000000UL // !!no comment, when calibrate!!
#define F_MIN        130700000UL        // !!comment, when calibrate!! Lower frequency limit
#define F_MAX        190700000UL      // Upper frequency limit
// !LOWER AND UPPER FREQUENCIES

/**************************************************************************
   EEPROM data locations
**************************************************************************/
#define EE_SAVED_RADIX  0   // Stepsize pointer
#define EE_SAVED_AFREQ  4   // Actual Tx Frequency (CLK0)
//#define EE_SAVED_BFREQ  8   // BFO (IF) Frequency  (CLK2)
#define EE_SAVED_XFREQ  12  // X-tal frequency  (25 or 27 MHz)
#define EE_SAVED_OFSET  16  // store correction
#define EE_SAVED_CALBR  20  // calibrated indicator

////////CHANNELS/////////////////

//#define CHANNEL1  24   //Channels in EEPROM
//#define CHANNEL200  824
#define CHANNEL_MAX  200 // just number of max channel. not eeprom.


////////// !CHANNELS///////////////


Adafruit_SSD1306 display(OLED_RESET);
Si5351 si5351;
Rotary r = Rotary(ENCODER_A, ENCODER_B);

/**************************************************************************
   Declarations
**************************************************************************/
volatile uint32_t bfo_f = 900000000ULL / SI5351_FREQ_MULT;    // CLK0 start IF
volatile uint32_t vfo_t = 1420000000ULL / SI5351_FREQ_MULT;   // CLK2 start Tx freq
volatile uint32_t vfo_r = vfo_t - bfo_f;                      // CLK1 start Rx freq
volatile uint32_t vfo_s = vfo_t;                              // Saved for RIT
uint32_t vco_c = 0;                                           // X-tal correction factor
uint32_t xt_freq;
long radix = 100L, old_radix = 100L;                          //start step size
boolean changed_f = 0, stepflag = 0, calflag = 0, modeflag = 0, ritset = 0;
boolean calibrate = 0;
byte  act_clk = 0, disp_txt = 0;

volatile uint32_t vfo_mem; // when we come back into "main mode" from read channels mode, we return into freq, we were.

//////////SCAN////////////////
boolean scan_flag;
boolean scan_scan_flag;
volatile uint32_t scan_start;
volatile uint32_t scan_end;
volatile uint32_t scan;
///////// !SCAN////////////////

/////////CHANNELS//////////////
byte channels_read; // number of channel
boolean channels_flag; // when read channel mode is on
boolean channels_auto_flag; // scan channels is on
byte last_channel; // scan channels
byte first_channel; // scan channels
boolean dop_flag;
boolean dop_dop_flag;
boolean write_flag; // write channels mode is on
boolean block_flag; // sleep mode, oled - off, encoder - is blocked for tuning
volatile uint16_t channels_mem; // place in EEPROM for channels
//////// !CHANNELS//////////////


////Prototypes///////

uint32_t scanScan(uint32_t vfo_t);
void channelsWrite();
void channelsRead();
byte scanChannels(byte read_channels);
void autoch();

//// ! Prototypes///////


/**************************************/
/* Interrupt service routine for      */
/* encoder frequency change           */
/**************************************/
ISR(PCINT2_vect) {
  char result = r.process();
  if(block_flag==false)
  {
  if (result == DIR_CW)
  {
    if ((scan_flag == false)&&(channels_flag==false)&&(write_flag==false))
    set_frequency(1);

    ////////////CHANNELS/////////
     if ((scan_flag == 0)&&((channels_flag==true)||(write_flag==true)))
      set_channels(1);
    ////////////!CHANNELS/////////  
  }
  else if (result == DIR_CCW)
  {
    if ((scan_flag == false)&&(channels_flag==false)&&(write_flag==false))
    set_frequency(-1);
    
    ////////////CHANNELS/////////
    if ((scan_flag == 0)&&((channels_flag==true)||(write_flag==true)))
      set_channels(-1);
      ////////!CHANNELS////////
  }
}
}


/**************************************/
/* Change the frequency               */
/* dir = 1    Increment               */
/* dir = -1   Decrement               */
/**************************************/
void set_frequency(short dir)
{
  switch (act_clk)
  {
    case 0:                 // Tx frequency
      if (dir == 1)
        vfo_t += radix;
      if (dir == -1)
        vfo_t -= radix;
      break;
    case 1:                 // Tx frequency (only if RIT is on)
      if (dir == 1)
        vfo_t += radix;
      if (dir == -1)
        vfo_t -= radix;
      break;
    case 2:                 // BFO frequency
      //if (dir == 1)
        //bfo_f += radix;
      //if (dir == -1)
        //bfo_f -= radix;
      break;
  }

  if (vfo_t > F_MAX)
    vfo_t = F_MIN;
  if (vfo_t < F_MIN)
    vfo_t = F_MAX;

  changed_f = 1;
}

////////////SCAN////////////
void set_scan_frequency(short dir)
{
  if (dir == 1)
    scan += radix;
  if (dir == -1)
    scan -= radix;
  if (scan > 180000000)
    scan = 180000000;
  if (scan < 120000000)
    scan = 120000000;
  changed_f = 1;
}
//////////// !SCAN////////////


////////////CHANNELS////////
void set_channels(short dir)
{
  if (dir == 1)
  {
    channels_read+=1;
  }
  if (dir == -1)
  {
    channels_read-=1;
  }
    
    if (channels_read > CHANNEL_MAX)
    channels_read = 1;
  if (channels_read < 1)
    channels_read = CHANNEL_MAX;
         
    changed_f = 1;
}
////////// !CHANNELS/////////




/**************************************/
/* Read the buttons with debouncing   */
/**************************************/
boolean get_button()
{
  if ((!digitalRead(ENCODER_BTN)) &&(channels_flag==false))           // Stepsize
  {
    delay(20);
    if ((!digitalRead(ENCODER_BTN))&&(channels_flag==false)) 
    {
      while (!digitalRead(ENCODER_BTN));
      stepflag = 1;
    }
  }
  else if (!digitalRead(Calibrbtn))         // Calibrate
  {
    delay(20);
    if (!digitalRead(Calibrbtn))
    {
      while (!digitalRead(Calibrbtn));
      calflag = 1;
    }
  }
  else if (!digitalRead(TX_Switch))         // Selection
  {
    //delay(20);
    if (!digitalRead(TX_Switch))
    {
      while (!digitalRead(TX_Switch));
      modeflag = 1;
    }
  }
  if (stepflag | calflag | modeflag) return 1;
  else return 0;
}


/**************************************/
/* Displays the frequency and stepsize*/
/**************************************/
void display_frequency()
{
  display.setTextSize(1);
  char LCDstr[12];
  char Mhz[5], Herz[12];
  int p, q;
  unsigned long freq;
  display.clearDisplay();

  switch (act_clk)
  {
    case 0:                               // Tx frequency
        freq = vfo_t - 10700000;  // !!comment, when calibrate!! (10.7 MHz - intermediate frequency ) 
      
        //freq=vfo_t;  // !!no comment, when calibrate!!
      
      break;
    case 1:                               // Tx frequency (Used in RIT Mode)
      freq = vfo_t;
      break;
    case 2:                               // MF frequency
      freq = bfo_f;
      break;
  }

  Herz[1] = '\0';                         // empty arry

  sprintf(LCDstr, "%ld", freq);           // convert freq to string
  p = strlen(LCDstr);                     // determine length

  strncpy(Mhz, LCDstr, (p - 6));          // get MHz digits (1-3)
  q = p - 6;
  Mhz[q] = '\0';                          // end with null character
  strcpy(Herz, LCDstr);                   // get Herz digits (6)
  strcpy(LCDstr + q, Herz + (q - 1));     // copy into LCDstr
  LCDstr[q] = '.';                        // decimal point

  display.setCursor(10, 0);
  display.println(LCDstr);


///////////// Svistelki i Perdelki ///////////////

  display.setCursor(9,10);
  
  //if(((vfo_t>=154700000)&&(vfo_t<155894000))||((vfo_t>=155906000)&&(vfo_t<156700000))&&((vfo_t!=156500000)&&(vfo_t!=156525000)))
  
  if((vfo_t>=154700000)&&(vfo_t<156700000))
  
    display.print("Ham Radio");
  
//if(vfo_t<=146700000)
  
  // display.print("Avia");

 // if((vfo_t>=155894000)&&(vfo_t<155906000))
  
   // display.print("Space");

  if((vfo_t>=161700000)&&(vfo_t<165700000))
  
    display.print("Railway");
  

  //if((vfo_t>=166700000)&&(vfo_t<174700000))
  
    //display.print("Sailing");
  

if (((vfo_t>=158700000)&&(vfo_t<159700000))||((vfo_t>=181700000)&&(vfo_t<183700000)))
  
    display.print("Police");

//if((vfo_t==156500000)||(vfo_t==156525000))
//{
//display.print("ISS");
//}
///////////// !Svistelki i Perdelki ///////////////

  

  /////////////SCAN////////
  if (scan_flag == true)
  {
    display.setCursor(100, 0);
    display.print("Run");
  }
  //////////!SCAN////////////
  
  ////CHANNELS//////


  if(dop_flag==true) //WHEN WRITE
{
   display.setCursor(100, 0);
   display.print("WRT");
}

  
if ((channels_flag==true)||(write_flag==true)) // PRINT WHEN CHANNELS ON
{
  display.setCursor(8, 20);
  display.setTextSize(1);
  display.print(F("Channel "));
  display.setCursor(69, 20);
  display.print(channels_read);
   
  
  display.display();
}
////!CHANNELS//////

else   
{
  display_settings();
}
}


/**************************************/
/* Displays step, mode and version    */
/**************************************/
void display_settings()
{
  // Stepsize
  display.setCursor(8, 20);
  display.setTextSize(1);
  display.print(F("Step:"));
  switch (radix)
  {
    
    case 100:
      display.println(F("  100Hz"));
      break;
    case 1000:
      display.println(F(" 1kHz"));
      break;
    case 5000:
      display.println(F(" 5kHz"));
      break;
    case 10000:
      display.println(F(" 10kHz"));
      break;
      case 25000:
      display.println(F(" 25kHz"));
      break;
    case 100000:
      display.println(F("100kHz"));
      break;
    case 1000000:
      display.println(F("  1MHz"));
      break;
  }
  display.display();
}



///////////////////////////SCAN (select)////////////////////////////////////////////////////////////////

uint32_t scanScan(uint32_t vfo_t)
{
  
if ((scan_flag==false)&&(scan_scan_flag==false)&&(digitalRead(scanfreq) == LOW))
{
  scan_scan_flag=true;
  scan_start=vfo_t;
  display.clearDisplay();
  display.setCursor(1, 0);
  display.setTextSize(1);
  display.print("START FREQ. REC");
  display.setCursor(10, 20);
  //display.print(scan_start-10700000);
  display.display();
  delay(1100);
  changed_f=1;
  display.clearDisplay();
  while (digitalRead(scanfreq) == LOW)
  {
  }
  return scan_start;
}

 if ((scan_scan_flag==true)&&(scan_flag==false)&&(digitalRead(scanfreq) == LOW))
{ 
  scan_scan_flag=false;
  scan_end=vfo_t;
  display.clearDisplay();
  display.setCursor(1, 00);
  display.setTextSize(1);
  display.print("FINAL FREQ. REC");
  display.setCursor(10, 20);
  //display.print(scan_end-10700000);
  display.display();
  delay(1100);
  changed_f=1;
  display.clearDisplay();
  while (digitalRead(scanfreq) == LOW)
  {
  }
  return scan_end;
}
 }
  
//////////// !SCAN (select)/////////////////////////////////////////////////////////////////////////////////



//////////////CHANNELS////////////////////////////////////////////////////////////////////////////////////////


//////WRITE CHANNELS/////////

void channelsWrite()
{
  while (digitalRead(channels_write_button) == LOW)
  {
  }
  delay(100);

  write_flag=true; // "channels mode"
  dop_flag=true; // for printing "REC"
  changed_f=1;
 
  while(write_flag==true)
  {
  
   if (changed_f) 
   {
  changed_f = 0;
  display_frequency();
   } 
  
  channels_mem=((4*channels_read)+20);

  // EXIT WITHOUT  WRITING
  if (digitalRead(channels_read_button) == LOW)  
  {
    delay(15);
    if (digitalRead(channels_read_button) == LOW)
  {
    display.clearDisplay();
    write_flag=false;
    dop_flag=false; 
    while(digitalRead(channels_read_button) == LOW)
    {
    }
    delay(100);
  }
  }
  //// !EXIT WITHOUT WRITING////////////


  ////WRITING and exit//////
  if ((digitalRead(channels_write_button) == LOW)&&(channels_flag==false)) // rec is impossible and unreasonable from read channels mode
  {
  delay(1500);
  if (digitalRead(channels_write_button) == LOW)
  {
  //////// write channels itself
  eeprom_write_dword((uint32_t *)channels_mem, vfo_t);
  // Print writing channels
  display.clearDisplay();
  display.setCursor(8, 20);
  display.setTextSize(1);
  display.print(F("REC! "));
 // display.print(channels_read); // On display, we are printing channels from 1 to 20, that's why "++";
  display.display();
  delay(1700);
  display.clearDisplay();
  write_flag=false;
  dop_flag=false;
  while(digitalRead(channels_write_button) == LOW)
    {
    }
    delay(200);
  }
  }
}
}
///////////// !CHANNELS WRITE//////////////////////////


/////////////////////////Scan channels//////////////////

 byte scanChannels(byte channels_read)
{
if ((channels_auto_flag==LOW)&&(dop_dop_flag==false)&&(digitalRead(scanfreq) == LOW))
{
  dop_dop_flag=true;
first_channel=channels_read;
  display.clearDisplay();
  display.setCursor(1, 0);
  display.setTextSize(1);
  display.print("FIRST CH. REC");
  display.setCursor(69, 20);
  display.print(first_channel);
  display.display();
  delay(1100);
  display.clearDisplay();
  changed_f=1;
  while(digitalRead(scanfreq) == LOW)
  {
  }
  return first_channel;
}

if ((channels_auto_flag==LOW)&&(dop_dop_flag==true)&&(digitalRead(scanfreq) == LOW))
{
   dop_dop_flag=false;
last_channel=channels_read;
  display.clearDisplay();
  display.setCursor(1, 0);
  display.setTextSize(1);
  display.print("LAST CH. REC ");
  display.setCursor(69, 20);
  display.print(last_channel);
  display.display();
  changed_f=1;
  delay(1100);
display.clearDisplay();
while(digitalRead(scanfreq) == LOW)
{
}
return last_channel;
}

}

///////////////////////// !Scan channels//////////////////



/////////////////////////CHANNELS READ////////////////////

void channelsRead()
{
  while (digitalRead(channels_read_button) == LOW)
  {
  }
  delay(100);
  dop_dop_flag=false;
  vfo_mem=vfo_t;         
  channels_flag=true; // now we are in "channels read mode"
  changed_f=1;
  display.clearDisplay();




///////////////////////////// READ CHANNELS LOOP ///////////////////////////////

  while(channels_flag==true)
  {


////////////////////SLEEP MODE /////////////////////////////////////////////////

// sleep mode, oled - off, encoder - is blocked for tuning
     if ((digitalRead(ENCODER_BTN)==LOW)&&(!channels_auto_flag))        
  {
    delay(20);
    if (digitalRead(ENCODER_BTN)==LOW)
    {
        block_flag=!block_flag;
  if(block_flag)
  {
    display.clearDisplay();
    display.setCursor(40, 10);
    display.print("SLEEP");
    display.display();
    delay(1200);
  display.ssd1306_command(174); //command AE - display OFF

set_sleep_mode(SLEEP_MODE_STANDBY);
sleep_mode();
  
  }
  if(!block_flag)
  display.ssd1306_command(175); //command AF - display ON
        
        changed_f=1;
         while(digitalRead(ENCODER_BTN)==LOW)
         {
         }
        delay(100);
        }
    }
    
 //////////////////// ! SLEEP MODE /////////////////////////////////////////////////



//////////////////// WRITE MODE /////////////////////////////////////////////////

 if ((digitalRead(channels_write_button) == LOW) && (scan_flag == false)&&(!block_flag))
  {
    delay(15);
    if (digitalRead(channels_write_button) == LOW)
    {
      channelsWrite();
      changed_f = 1;
    }
  }


//////////////////// ! WRITE MODE /////////////////////////////////////////////////


////////// CALL SELECT SCAN CHANNELS ////////////

 if ((digitalRead(scanfreq) == LOW)&&(channels_auto_flag==false)&&(!block_flag))
{ 
delay(35);
if (digitalRead(scanfreq) == LOW)
scanChannels (channels_read);
}

      
////////// ! CALL SELECT SCAN CHANNELS ////////////

// Repeat scan from first channel

  if (((channels_read>last_channel)||(channels_read<first_channel))&&(channels_auto_flag==true)&&(!block_flag))
  {
    changed_f=1;
    channels_read=first_channel;
    }
    
  
   if (changed_f) 
   {
  changed_f = 0; // MUST BE in the beginning only, otherwise with fast rotate of encoder will be problems
  channels_mem=((4*channels_read)+20);
  vfo_t = eeprom_read_dword((const uint32_t *)channels_mem);
  si5351.set_freq(vfo_t * SI5351_FREQ_MULT, SI5351_CLK0);
  //display_frequency();
 
   } 
   display_frequency();
  ///////// AUTO CHANNELS FUNCTION /////////////// 
  if ((digitalRead(scanstart) == LOW)&&(last_channel>first_channel))   // AUTO CHANNELS - ON
  {
  delay(25);
  if (digitalRead(scanstart) == LOW)
  {
    while((digitalRead(scanstart) == LOW))
    {
    }
    delay(45);
  (channels_auto_flag)=!(channels_auto_flag);
  
  if(channels_auto_flag==true)
  {
  //channels_read=first_channel; 
  scan_flag=true; //ENCODER-OFF
  Timer1.initialize(4000000);   // TIMER ON . 4 sec - is time hold on each channel
  Timer1.attachInterrupt(autoch);
  display.setCursor(100, 0);
    display.print("Run");
    changed_f=1;
  }
  if(channels_auto_flag==false)
  {
    TCCR1B &= ~(_BV(CS10) | _BV(CS11) | _BV(CS12));   // TIMER - STOP (when exit from "auto channels")
    changed_f=1;
    scan_flag=false;
  }
  }
  }

   
//////////////// !AUTO CHANNELS /////////////////////


///////EXIT FROM CHANNELS MODE
  if ((digitalRead(channels_read_button) == LOW)&&(channels_auto_flag==false)&&(!block_flag))
  {
  delay(15);
  if ((digitalRead(channels_read_button) == LOW)&&(channels_auto_flag==false)&&(!block_flag))
  {
    channels_flag=false;
    vfo_t=vfo_mem; // return to frequency before "channels mode" 
    while(digitalRead(channels_read_button) == LOW)
    {
    }
    delay(100);
  }
  }
  
}

}
///////// !EXIT FROM CHANNELS MODE//////////////

//////////////// !CHANNELS////////////////////////////////////////////////////////////////




/////////////// INTERRUPTS /////////////

void autoch()  //TIMER INTERRUPT FUNCTION (for "auto channels" and scan)
{
  if(channels_auto_flag==true) //AUTO CHANNELS
  {
  channels_read++;
  changed_f=1;
  }
  if((scan_flag==true)&&(radix!=100)&&(radix!=1000)&&(channels_auto_flag==false)) //SCAN 
  {
  vfo_t = vfo_t + radix;
  if (vfo_t >= scan_end)
       vfo_t = scan_start;
  changed_f = 1;
  }
}

void pin_ISR() 
{
}

/////////////// ! INTERRUPTS /////////////







/**************************************/
/*            S E T U P               */
/**************************************/
void setup()
{
  Serial.begin(115200);
  Wire.begin();
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);  // initialize with the I2C addr 0x3C (for the 128x64)

  vco_c = 0;
  if (eeprom_read_dword((const uint32_t *)EE_SAVED_CALBR) == 0x60)  {
    vco_c = eeprom_read_dword((const uint32_t *)EE_SAVED_OFSET);
  }
  xt_freq = SI5351_XTAL_FREQ + vco_c;

  //initialize the Si5351
  si5351.set_correction(27000000,0); // Set to zero because I'm using an other calibration method
  si5351.init(SI5351_CRYSTAL_LOAD_8PF, xt_freq, 0); //If you're using a 27Mhz crystal, put in 27000000 instead of 0
  // 0 is the default crystal frequency of 25Mhz.
  si5351.set_pll(SI5351_PLL_FIXED, SI5351_PLLA);

  // Set CLK0 to output the starting "vfo" frequency as set above by vfo = ?
  si5351.set_freq((vfo_t * SI5351_FREQ_MULT), SI5351_CLK0);
  
  si5351.drive_strength(SI5351_CLK0, SI5351_DRIVE_8MA);     // Output current - 8 MA

  si5351.output_enable(SI5351_CLK1, 0);  // CLK1, CLK2 - off
  si5351.output_enable(SI5351_CLK2, 0);

  // Clear the buffer.
  display.clearDisplay();
  display.display();

  // text display tests
  display.setTextSize(1);
  display.setTextColor(WHITE);

  // Encoder setup
  pinMode(ENCODER_BTN, INPUT_PULLUP);
  PCICR |= (1 << PCIE2);                       // Enable pin change interrupt for the encoder
  PCMSK2 |= (1 << PCINT18) | (1 << PCINT19);

  sei();

  // Pin Setup
  pinMode(Calibrbtn, INPUT_PULLUP);   // Calibrate
  pinMode(RIT_Switch, INPUT_PULLUP);  // RIT Switch  shouldn't delete this and next line for correct work to don't change original code
  pinMode(TX_Switch, INPUT_PULLUP);   // Select TRx or BFO

  //MY PINS 9...12
  pinMode(9, INPUT_PULLUP);
  pinMode(10, INPUT_PULLUP);
  pinMode(11, INPUT_PULLUP);
  pinMode(12, INPUT_PULLUP);


  /////////SCAN//////////
  scan_flag = false;
  scan_scan_flag=false;
  scan_start = 130700000;
  scan_end = 190700000;
  //////////!SCAN////////


///////////////////////WRITE to EEPROM EMPTY 1-200 CHANNELS//////////

//channels_mem=24;
//while(channels_mem<824)
//{  
//eeprom_write_dword((uint32_t *)channels_mem, 155700000); // write 145 MHz to 1-200 channels
//channels_mem+=4;
//}

/////////////////////// !WRITE to EEPROM EMPTY 1-200 CHANNELS//////////

  
  ////////CHANNELS//////
  channels_flag=false;
  channels_auto_flag=false;
  dop_flag=false;
  dop_dop_flag=false;
  channels_read=2; // goto 2 channel
  first_channel=1;
  last_channel=200;
  write_flag=false;
  block_flag=false;

  /////// !CHANNELS/////

  vfo_t=160700000;  //start freq., mean 150MHz
  radix=25000; // start step 25KHz

 changed_f=1;

 attachInterrupt(0, pin_ISR, CHANGE);
 attachInterrupt(1, pin_ISR, CHANGE);

}

/**************************************/
/*             L O O P                */
/**************************************/

void loop()
{

  ////////////SCAN///////////////////////////////////////////////////////////////////////////////
  
  if ((digitalRead(scanfreq) == LOW) && (scan_flag == false))
  {
    delay(35);
    if ((digitalRead(scanfreq) == LOW) && (scan_flag == false))
      scanScan(vfo_t);  // scan select function
  }

//////////////////////////////////////////////////////////////
  
//Scan itself
  if (scan_flag == true) 
  {   
    if((radix==100)||(radix==1000))
    {
       if (vfo_t >= scan_end)
       vfo_t = scan_start;
       
    vfo_t=vfo_t+radix;
    changed_f = 1;
    }
  }
/////////////////////////////////////////////////////////////

  
//stop scan
  if ((digitalRead(scanstart) == LOW) && (scan_flag == true))
  {
    delay(15);
    if ((digitalRead(scanstart) == LOW) && (scan_flag == true))
    {
      scan_flag = false;
      changed_f = 1;
      
      TCCR1B &= ~(_BV(CS10) | _BV(CS11) | _BV(CS12)); // stop timer
      
     while (digitalRead(scanstart) == LOW)
      {
      }
      delay(40);
    }
  }

//start scan
  if ((digitalRead(scanstart) == LOW) && (scan_flag == false)&& ((scan_start) < (scan_end))&&(channels_flag==false))
  {
    delay(15);
    if ((digitalRead(scanstart) == LOW) && (scan_flag == false))
    {
      if((vfo_t>scan_end)||(vfo_t<scan_start))
      vfo_t=scan_start;
      
      Timer1.initialize(2000000);   // TIMER ON . 2.0 sec - is time hold on each scan step
      Timer1.attachInterrupt(autoch);   
      changed_f = 1;
      while (digitalRead(scanstart) == LOW)
      {
      }
      delay(50);
      scan_flag = true;
    }
  }
  //////////// !SCAN/////////////////////////////////////////////////////////////////////////////////////////////



  ////////////CHANNELS///////////////////////////////////////////////////////////////////////////////////////////
  
  if ((digitalRead(channels_write_button) == LOW) && (scan_flag == false))
  {
    delay(15);
    if ((digitalRead(channels_write_button) == LOW) && (scan_flag == false))
    {
      channelsWrite();
      changed_f = 1;
    }
  }

  if ((digitalRead(channels_read_button) == LOW) && (scan_flag == false))
  {
    delay(15);
    if ((digitalRead(channels_read_button) == LOW) && (scan_flag == false))
    {
      channelsRead();
      changed_f = 1;
    }
  }
  ///////////////!CHANNELS///////////////////////////////////////////////////////////////////////////////////////////
  



 if (disp_txt == 4) {
    delay(3000);                  // Display calibration OK and wait 3 seconds
    disp_txt = 0;
  }
// Update the display if the frequency has been changed
  if (changed_f) 
  {
    changed_f = 0; // MUST BE in the beginning only, otherwise may be problems with fast rotate of encoder       
    if (act_clk == 0 && !calibrate)
    si5351.set_freq((vfo_t * SI5351_FREQ_MULT), SI5351_CLK0); 
    
   // display_frequency(); 
    
    disp_txt = 0;  // Clear line                   
  }

display_frequency();

  if (get_button()) {
    if (stepflag) {                 // Stepsize button

      switch (radix)
      {
      case 100:
        radix = 1000;
        break;
      case 1000:
        radix = 5000;
        break;
      case 5000:
        radix = 10000;
        break;
      case 10000:
        radix = 25000;
        break;
        case 25000:
        radix = 100000;
        break;
      case 100000:
        radix = 1000000;
        break;
      case 1000000:
        radix = 100;
        break;       
      }
  changed_f = 1;
      stepflag  = 0;
    }
    else if (modeflag)  {         // Mode button
      if (act_clk == 0) act_clk = 2; else act_clk = 0;
    //  eeprom_write_dword((uint32_t *)EE_SAVED_BFREQ, bfo_f);
      modeflag = 0;  
      disp_txt = 0;                                 // Clear line
    }

   else if (calflag) 
   {                             // Calibrate button
      if (!digitalRead(RIT_Switch)){                // RIT is on
        disp_txt = 1;                               // Message RIT off
      }
       if (act_clk == 2){                       // BFO mode on
        disp_txt = 2;                               // Message BFO off        
      }
      else if (!calibrate)  {                       // Start calibrate
        vfo_s = vfo_t;                              // Save actual freq
        old_radix = radix;                          // and stepsize
        vfo_t = SI5351_XTAL_FREQ;                   // en set to default x-tal
        disp_txt = 3;                               // Message Calibrate
        calibrate = 1;
        radix = 100;                                 // Set to 100 Hz        
        si5351.set_freq((vfo_t * SI5351_FREQ_MULT), SI5351_CLK0); // Set CLK0
      }
      else if (calibrate) {                         // after tuning x-tal freq
        calibrate = 0;
        vco_c = vfo_t - SI5351_XTAL_FREQ;           // difference
        vfo_t = vfo_s;                              // restore freq
        radix = old_radix;                          // and stepsize
        disp_txt = 4;                               // Message Calibrate OK
        
        eeprom_write_dword((uint32_t *)EE_SAVED_OFSET, vco_c);        // store correction
        xt_freq = SI5351_XTAL_FREQ + vco_c;                           // Calibrated x-tal freq
        eeprom_write_dword((uint32_t *)EE_SAVED_CALBR, 0x60);         // Calibrated
        si5351.init(SI5351_CRYSTAL_LOAD_8PF, xt_freq, 0);                // Initialize
        si5351.set_pll(SI5351_PLL_FIXED, SI5351_PLLA);        
       // si5351.set_freq(bfo_f * SI5351_FREQ_MULT, SI5351_CLK2);   // correct BFO frequency
        si5351.set_freq(vfo_t * SI5351_FREQ_MULT, SI5351_CLK0);   // Correct Tx freq
       // if (vfo_t <= bfo_f) vfo_r = vfo_t + bfo_f;                                  // Upper / lower mixing
         // else vfo_r = vfo_t - bfo_f;
       // si5351.set_freq(vfo_r * SI5351_FREQ_MULT, SI5351_CLK1);   // correct Rx frequency
      }
    calflag = 0;
    }
  }    
 
  
} // end while loop



////////////////// !RECEIVER ARMSTRONG INITIAL ///////////////////////////////////////////////////////////////////////////////
